@@ -20,7 +20,7 @@
 #' @param stepdown      logical, if TRUE a stepdown procedure is applied
 #' @param seed          seed for the Gaussian simulations
 #'
-#' @return Returns a line vector containing estimated fwer, estimated fdr, estimated power, estimated true discovery rate.
+#' @return Returns a line vector containing estimated values for fwer, fdr, sensitivity, specificity and accuracy.
 #'
 #' @importFrom MASS mvrnorm
 #' @export
@@ -37,18 +37,18 @@
 #' n <- 100
 #' p <- 10
 #' corr_theo <- diag(1,p)
+#' corr_theo[1,3] <- 0.5
+#' corr_theo[3,1] <- 0.5
 #' alpha <- 0.05
-#' res <- SimuFwer(corr_theo,n,Nsimu,alpha,stat_test='empirical',method='Bonferroni',stepdown=FALSE)
-SimuFwer <- function(corr_theo,n=100,Nsimu=1,alpha=0.05,stat_test='empirical',method='MaxTinfty',Nboot=1000,stepdown=TRUE,seed=NULL){
-
-  if((stat_test=='gaussian')&(method=='MaxTinfty')){
-        stop('MaxTinfty procedure is not implemented for Gaussian type statistics.\n')
-  }
-
+#' SimuFwer(corr_theo,n,Nsimu,alpha,stat_test='empirical',method='Bonferroni',stepdown=FALSE)
+SimuFwer <- function(corr_theo,n=100,Nsimu=1,alpha=0.05,stat_test='empirical',method='Sidak',Nboot=1000,stepdown=TRUE,seed=NULL){
 
+  if(sum(stat_test==c('empirical','fisher','student','2nd.order'))==0){ stop('Wrong value for stat_test.')}
+  if(sum(method==c('Bonferroni','Sidak','BootRW','MaxTinfty'))==0){ stop('Wrong value for method.')}
   if(!is.null(seed)){ set.seed(seed) }
-  p <- nrow(corr_theo)
 
+  p <- nrow(corr_theo)
+  m <- p*(p-1)/2
   corr_theo_vect <- vectorize(corr_theo)
   H1 <- (corr_theo_vect!=0)
 
@@ -57,20 +57,21 @@ SimuFwer <- function(corr_theo,n=100,Nsimu=1,alpha=0.05,stat_test='empirical',me
 
 	false_positive <- array(0,dim=c(Nsimu))
 	true_positive <- array(0,dim=c(Nsimu))
-    nb_rejet <- array(0,dim=c(Nsimu))
+  nb_rejet <- array(0,dim=c(Nsimu))
 	for(nsimu in 1:Nsimu){	
 	   data <- data_all[(1:n)+n*(nsimu-1), ]
-	   rejet <- ApplyFwerCor(data,alpha,stat_test,method,Nboot,stepdown,vect=TRUE)
-	   true_positive[nsimu] <- ifelse(sum(H1)==0,0,sum(H1[rejet])/sum(H1))
-       false_positive[nsimu] <- ifelse(sum(rejet)==0,0,(sum(rejet)-sum(H1[rejet]))/sum(rejet))
-       nb_rejet[nsimu] <- sum(rejet)
+	   rejet <- ApplyFwerCor(data,alpha,stat_test,method,Nboot,stepdown,vect=TRUE,logical=TRUE)
+	   true_positive[nsimu] <- sum(H1[rejet])
+     nb_rejet[nsimu] <- sum(rejet)
 	}
-
-  # return fwer, fdr, power, tdr
-  res  <- c(mean(false_positive>0),mean(false_positive),mean(true_positive),mean(ifelse(nb_rejet==0,0,true_positive/nb_rejet*sum(H1))))
-  names(res) <- c('fwer','fdr','power','tdr')
+  false_positive <- nb_rejet-true_positive
+  true_negative <- m-sum(H1)-false_positive
+  m1 <- rep(sum(H1),Nsimu)
+  
+  # return fwer, fdr, sensitivity, specificity, accuracy
+  res  <- c(mean(false_positive>0),mean(ifelse(nb_rejet==0,0,false_positive/nb_rejet)),mean(ifelse(m1==0,0,true_positive/sum(H1))),mean(ifelse(m1==m,0,true_negative/(m-sum(H1)))),mean((true_positive+true_negative)/m))
+  names(res) <- c('fwer','fdr','sensitivity','specificity','accuracy')
   
-
   return(res)
 
 
@@ -95,7 +96,7 @@ SimuFwer <- function(corr_theo,n=100,Nsimu=1,alpha=0.05,stat_test='empirical',me
 #' @param stepdown      logical, if TRUE a stepdown procedure is applied
 #' @param seed          seed for the Gaussian simulations
 #'
-#' @return Returns a line vector containing estimated fwer, estimated fdr, estimated power, estimated true discovery rate.
+#' @return Returns a line vector containing estimated values for fwer, fdr, sensitivity, specificity and accuracy.
 #'
 #' @references  Drton, M., & Perlman, M. D. (2007). Multiple testing and error control in Gaussian graphical model selection. Statistical Science, 22(3), 430-449.
 #' @references  Roux, M. (2018). Graph inference by multiple testing with application to Neuroimaging, Ph.D., Université Grenoble Alpes, France, https://tel.archives-ouvertes.fr/tel-01971574v1.
@@ -107,21 +108,20 @@ SimuFwer <- function(corr_theo,n=100,Nsimu=1,alpha=0.05,stat_test='empirical',me
 #' n <- 50
 #' p <- 10
 #' corr_theo <- diag(1,p)
+#' corr_theo[1,3] <- 0.5
+#' corr_theo[3,1] <- 0.5
 #' alpha <- 0.05
-#' res <- SimuFwer_oracle(corr_theo,n,Nsimu,alpha,stat_test='empirical',stepdown=FALSE,Nboot=100)
+#' SimuFwer_oracle(corr_theo,n,Nsimu,alpha,stat_test='empirical',stepdown=FALSE,Nboot=100)
 SimuFwer_oracle <- function(corr_theo,n=100,Nsimu=1,alpha=0.05,stat_test='empirical',method='MaxTinfty',Nboot=1000,stepdown=TRUE,seed=NULL){
 
+  if(sum(stat_test==c('empirical','fisher','student','2nd.order'))==0){ stop('Wrong value for stat_test.')}
   if(method!='MaxTinfty'){
        stop('The only oracle procedure available is MaxTinfty.\n')
   }
-  if(stat_test=='gaussian'){
-       stop('MaxTinfty procedure is not implemented for Gaussian type statistics.\n')
-  }
-
-
   if(!is.null(seed)){ set.seed(seed) }
-  p <- nrow(corr_theo)
 
+  p <- nrow(corr_theo)
+  m <- p*(p-1)/2
   corr_theo_vect <- vectorize(corr_theo)
   H1 <- (corr_theo_vect!=0)
 
@@ -130,20 +130,21 @@ SimuFwer_oracle <- function(corr_theo,n=100,Nsimu=1,alpha=0.05,stat_test='empiri
 
 	false_positive <- array(0,dim=c(Nsimu))
 	true_positive <- array(0,dim=c(Nsimu))
-    nb_rejet <- array(0,dim=c(Nsimu))
+  nb_rejet <- array(0,dim=c(Nsimu))
 	for(nsimu in 1:Nsimu){	
 	   data <- data_all[(1:n)+n*(nsimu-1), ]
-	   rejet <- ApplyFwerCor_oracle(data,corr_theo,alpha,stat_test,method,Nboot,stepdown,vect=TRUE)
-	   true_positive[nsimu] <- ifelse(sum(rejet)==0,0,sum(H1[rejet])/sum(rejet))
-       false_positive[nsimu] <- ifelse(sum(rejet)==0,0,(sum(rejet)-sum(H1[rejet]))/sum(rejet))
-       nb_rejet[nsimu] <- sum(rejet)
+	   rejet <- ApplyFwerCor(data,alpha,stat_test,method,Nboot,stepdown,vect=TRUE,logical=TRUE)
+	   true_positive[nsimu] <- sum(H1[rejet])
+	   nb_rejet[nsimu] <- sum(rejet)
 	}
-
-  # return fwer, fdr, power, tdr
-res  <- c(mean(false_positive>0),mean(false_positive),mean(true_positive),mean(ifelse(nb_rejet==0,0,true_positive/nb_rejet*sum(H1))))
-  names(res) <- c('fwer','fdr','power','tdr')
-
-	
+  false_positive <- nb_rejet-true_positive
+  true_negative <- m-sum(H1)-false_positive
+  m1 <- rep(sum(H1),Nsimu)
+  
+  # return fwer, fdr, sensitivity, specificity, accuracy
+  res  <- c(mean(false_positive>0),mean(ifelse(nb_rejet==0,0,false_positive/nb_rejet)),mean(ifelse(m1==0,0,true_positive/sum(H1))),mean(ifelse(m1==m,0,true_negative/(m-sum(H1)))),mean((true_positive+true_negative)/m))
+  names(res) <- c('fwer','fdr','sensitivity','specificity','accuracy')
+  
   return(res)
 }
 
@@ -169,7 +170,8 @@ res  <- c(mean(false_positive>0),mean(false_positive),mean(true_positive),mean(i
 #' @param Nboot       number of iterations for Monte-Carlo of bootstrap quantile evaluation
 #' @param seed        seed for the Gaussian simulations
 #'
-#' @return Returns a line vector containing estimated fwer, estimated fdr, estimated power, estimated true discovery rate.
+#' @return Returns a line vector containing estimated values for fwer, fdr, sensitivity, specificity and accuracy.
+#' 
 #' @importFrom MASS mvrnorm
 #' @export
 #'
@@ -183,34 +185,40 @@ res  <- c(mean(false_positive>0),mean(false_positive),mean(true_positive),mean(i
 #' n <- 100
 #' p <- 10
 #' corr_theo <- diag(1,p)
+#' corr_theo[1,3] <- 0.5
+#' corr_theo[3,1] <- 0.5
 #' alpha <- 0.05
-#' res <- SimuFdr(corr_theo,n,Nsimu,alpha,stat_test='empirical',method='LCTnorm')
+#' SimuFdr(corr_theo,n,Nsimu,alpha,stat_test='empirical',method='LCTnorm')
 SimuFdr <- function(corr_theo,n=100,Nsimu=1,alpha=0.05,stat_test='empirical',method='LCTnorm',Nboot=1000,seed=NULL){
 
+  if(sum(stat_test==c('empirical','fisher','student','2nd.order'))==0){ stop('Wrong value for stat_test.')}
+  if(sum(method==c('LCTnorm','LCTboot','BH','BHboot'))==0){ stop('Wrong value for method.')}
   if(!is.null(seed)){ set.seed(seed) }
-  p <- nrow(corr_theo)
 
+  p <- nrow(corr_theo)
+  m <- p*(p-1)/2
   corr_theo_vect <- vectorize(corr_theo)
   H1 <- (corr_theo_vect!=0)
 
   # we simulate all data outside the loop to improve time
   data_all <- mvrnorm(n*Nsimu,rep(0,p),corr_theo)
 
-	false_positive <- array(0,dim=c(Nsimu))
 	true_positive <- array(0,dim=c(Nsimu))
-    nb_rejet <- array(0,dim=c(Nsimu))
+  nb_rejet <- array(0,dim=c(Nsimu))
 	for(nsimu in 1:Nsimu){	
 	   data <- data_all[(1:n)+n*(nsimu-1), ]
 	   rejet <- ApplyFdrCor(data,alpha,stat_test,method,Nboot,vect=TRUE)
-	   true_positive[nsimu] <- ifelse(sum(H1)==0,0,sum(H1[rejet])/sum(H1))
-       false_positive[nsimu] <- ifelse(sum(rejet)==0,0,(sum(rejet)-sum(H1[rejet]))/sum(rejet))
-       nb_rejet[nsimu] <- sum(rejet)
+	   true_positive[nsimu] <- sum(H1[rejet])
+	   nb_rejet[nsimu] <- sum(rejet)
 	}
-
-  # return fwer, fdr, power, tdr
-  res  <- c(mean(false_positive>0),mean(false_positive),mean(true_positive),mean(ifelse(nb_rejet==0,0,true_positive/nb_rejet*sum(H1))))
-  names(res) <- c('fwer','fdr','power','tdr')
-
+  false_positive <- nb_rejet-true_positive
+  true_negative <- m-sum(H1)-false_positive
+  m1 <- rep(sum(H1),Nsimu)
+  
+  # return fwer, fdr, sensitivity, specificity, accuracy
+  res  <- c(mean(false_positive>0),mean(ifelse(nb_rejet==0,0,false_positive/nb_rejet)),mean(ifelse(m1==0,0,true_positive/sum(H1))),mean(ifelse(m1==m,0,true_negative/(m-sum(H1)))),mean((true_positive+true_negative)/m))
+  names(res) <- c('fwer','fdr','sensitivity','specificity','accuracy')
+    
   return(res)
 }
 
